﻿/*
 * Preference.cs
 * Copyright (c) 2008-2009 kbinani
 *
 * This file is part of Boare.Cadencii.
 *
 * Boare.Cadencii is free software; you can redistribute it and/or
 * modify it under the terms of the BSD License.
 *
 * Boare.Cadencii is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
using System;
using System.Drawing;
using System.IO;
using System.Collections.Generic;
using System.Windows.Forms;

using Boare.Lib.Vsq;
using Boare.Lib.AppUtil;
using Boare.Lib.Media;

using bocoree;

namespace Boare.Cadencii {

    partial class Preference : Form {
        Font m_base_font;
        Font m_screen_font;
        Font m_counter_font;
        List<string> m_program_change = null;
        private Platform m_platform = Platform.Windows;
        private List<SingerConfig> m_utau_singers = new List<SingerConfig>();

        public Preference() {
            InitializeComponent();
            ApplyLanguage();

            comboVibratoLength.Items.Clear();
            foreach ( DefaultVibratoLength dvl in Enum.GetValues( typeof( DefaultVibratoLength ) ) ) {
                comboVibratoLength.Items.Add( (int)dvl + "" );
            }
            comboVibratoLength.SelectedIndex = 1;

            comboAutoVibratoMinLength.Items.Clear();
            foreach ( AutoVibratoMinLength avml in Enum.GetValues( typeof( AutoVibratoMinLength ) ) ) {
                comboAutoVibratoMinLength.Items.Add( (int)avml + "" );
            }
            comboAutoVibratoMinLength.SelectedIndex = 0;

            comboAutoVibratoType.Items.Clear();
            foreach ( VibratoType vt in Enum.GetValues( typeof( VibratoType ) ) ) {
                comboAutoVibratoType.Items.Add( VibratoTypeUtil.ToString( vt ) );
            }
            comboAutoVibratoType.SelectedIndex = 0;

            comboDynamics.Items.Clear();
            comboAmplitude.Items.Clear();
            comboPeriod.Items.Clear();
            foreach ( ClockResolution cr in ClockResolution.GetEnumerator() ) {
                comboDynamics.Items.Add( cr.ToString() );
                comboAmplitude.Items.Add( cr.ToString() );
                comboPeriod.Items.Add( cr.ToString() );
            }
            comboDynamics.SelectedIndex = 0;
            comboAmplitude.SelectedIndex = 0;
            comboPeriod.SelectedIndex = 0;

            comboLanguage.Items.Clear();
            string[] list = Messaging.GetRegisteredLanguage();
            int index = 0;
            comboLanguage.Items.Add( "Default" );
            int count = 0;
            foreach ( string s in list ) {
                count++;
                comboLanguage.Items.Add( s );
                if ( s == Messaging.Language ) {
                    index = count;
                }
            }
            comboLanguage.SelectedIndex = index;

            SingerConfig[] dict = VocaloSysUtil.getSingerConfigs2();
            m_program_change = new List<string>();
            comboDefualtSinger.Items.Clear();
            foreach ( SingerConfig kvp in dict ) {
                m_program_change.Add( kvp.VOICENAME );
                comboDefualtSinger.Items.Add( kvp.VOICENAME );
            }
            comboDefualtSinger.Enabled = (comboDefualtSinger.Items.Count > 0);
            if ( comboDefualtSinger.Items.Count > 0 ) {
                comboDefualtSinger.SelectedIndex = 0;
            }

            comboPlatform.Items.Clear();
#if DEBUG
            Common.DebugWriteLine( "Preference.ctor()" );
            Common.DebugWriteLine( "    Environment.OSVersion.Platform=" + Environment.OSVersion.Platform );
#endif
#if !DEBUG
            PlatformID platform = Environment.OSVersion.Platform;
            if ( platform == PlatformID.Win32NT ||
                 platform == PlatformID.Win32S ||
                 platform == PlatformID.Win32Windows ||
                 platform == PlatformID.WinCE ) {
                comboPlatform.Items.Add( Platform.Windows + "" );
                comboPlatform.Enabled = false;
                chkCommandKeyAsControl.Enabled = false;
            } else {
#endif
                foreach ( Platform p in Enum.GetValues( typeof( Platform ) ) ) {
                    comboPlatform.Items.Add( p + "" );
                }
#if !DEBUG
            }
#endif

            comboMidiInPortNumber.Items.Clear();
            bocoree.MIDIINCAPS[] midiins = MidiInDevice.GetMidiInDevices();
            for ( int i = 0; i < midiins.Length; i++ ) {
                comboMidiInPortNumber.Items.Add( midiins[i] );
            }
            if ( comboMidiInPortNumber.Items.Count == 0 ) {
                comboMidiInPortNumber.Enabled = false;
            } else {
                comboMidiInPortNumber.Enabled = true;
            }

            txtVOCALOID1.Text = VocaloSysUtil.getDllPathVsti1();
            txtVOCALOID2.Text = VocaloSysUtil.getDllPathVsti2();
        }

        public bool SelfDeRomantization {
            get {
                return chkTranslateRoman.Checked;
            }
            set {
                chkTranslateRoman.Checked = value;
            }
        }

        public bool UseCustomFileDialog {
            get {
                return chkUseCustomFileDialog.Checked;
            }
            set {
                chkUseCustomFileDialog.Checked = value;
            }
        }

        public bool InvokeWithWine {
            get {
                return chkInvokeWithWine.Checked;
            }
            set {
                chkInvokeWithWine.Checked = value;
            }
        }

        public int MidiInPort {
            get {
                if ( comboMidiInPortNumber.Enabled ) {
                    if ( comboMidiInPortNumber.SelectedIndex >= 0 ) {
                        return comboMidiInPortNumber.SelectedIndex;
                    } else {
                        return 0;
                    }
                } else {
                    return -1;
                }
            }
            set {
                if ( comboMidiInPortNumber.Enabled ) {
                    if ( 0 <= value && value < comboMidiInPortNumber.Items.Count ) {
                        comboMidiInPortNumber.SelectedIndex = value;
                    } else {
                        comboMidiInPortNumber.SelectedIndex = 0;
                    }
                }
            }
        }

        public bool CurveVisibleVel {
            get {
                return chkVel.Checked;
            }
            set {
                chkVel.Checked = value;
            }
        }

        public bool CurveVisibleAccent {
            get {
                return chkAccent.Checked;
            }
            set {
                chkAccent.Checked = value;
            }
        }

        public bool CurveVisibleDecay {
            get {
                return chkDecay.Checked;
            }
            set {
                chkDecay.Checked = value;
            }
        }

        public bool CurveVisibleVibratoRate {
            get {
                return chkVibratoRate.Checked;
            }
            set {
                chkVibratoRate.Checked = value;
            }
        }

        public bool CurveVisibleVibratoDepth {
            get {
                return chkVibratoDepth.Checked;
            }
            set {
                chkVibratoDepth.Checked = value;
            }
        }

        public bool CurveVisibleDyn {
            get {
                return chkDyn.Checked;
            }
            set {
                chkDyn.Checked = value;
            }
        }

        public bool CurveVisibleBre {
            get {
                return chkBre.Checked;
            }
            set {
                chkBre.Checked = value;
            }
        }

        public bool CurveVisibleBri {
            get {
                return chkBri.Checked;
            }
            set {
                chkBri.Checked = value;
            }
        }

        public bool CurveVisibleCle {
            get {
                return chkCle.Checked;
            }
            set {
                chkCle.Checked = value;
            }
        }

        public bool CurveVisibleOpe {
            get {
                return chkOpe.Checked;
            }
            set {
                chkOpe.Checked = value;
            }
        }

        public bool CurveVisiblePor {
            get {
                return chkPor.Checked;
            }
            set {
                chkPor.Checked = value;
            }
        }

        public bool CurveVisibleGen {
            get {
                return chkGen.Checked;
            }
            set {
                chkGen.Checked = value;
            }
        }

        public bool CurveVisiblePitch {
            get {
                return chkPitch.Checked;
            }
            set {
                chkPitch.Checked = value;
            }
        }

        public bool CurveVisibleFx2Depth {
            get {
                return chkFx2Depth.Checked;
            }
            set {
                chkFx2Depth.Checked = value;
            }
        }

        public bool CurveVisibleHarmonics {
            get {
                return chkHarmonics.Checked;
            }
            set {
                chkHarmonics.Checked = value;
            }
        }

        public bool CurveVisibleReso1 {
            get {
                return chkReso1.Checked;
            }
            set {
                chkReso1.Checked = value;
            }
        }

        public bool CurveVisibleReso2 {
            get {
                return chkReso2.Checked;
            }
            set {
                chkReso2.Checked = value;
            }
        }
        
        public bool CurveVisibleReso3 {
            get {
                return chkReso3.Checked;
            }
            set {
                chkReso3.Checked = value;
            }
        }

        public bool CurveVisibleReso4 {
            get {
                return chkReso4.Checked;
            }
            set {
                chkReso4.Checked = value;
            }
        }

        public bool CurveSelectingQuantized {
            get {
                return chkCurveSelectingQuantized.Checked;
            }
            set {
                chkCurveSelectingQuantized.Checked = value;
            }
        }

        public bool PlayPreviewWhenRightClick {
            get {
                return chkPlayPreviewWhenRightClick.Checked;
            }
            set {
                chkPlayPreviewWhenRightClick.Checked = value;
            }
        }

        public int MouseHoverTime {
            get {
                return (int)numMouseHoverTime.Value;
            }
            set {
                numMouseHoverTime.Value = value;
            }
        }

        public int PxTrackHeight {
            get {
                return (int)numTrackHeight.Value;
            }
            set {
                numTrackHeight.Value = value;
            }
        }

        public bool KeepLyricInputMode {
            get {
                return chkKeepLyricInputMode.Checked;
            }
            set {
                chkKeepLyricInputMode.Checked = value;
            }
        }

        public Platform Platform {
            get {
                return m_platform;
            }
            set {
                m_platform = value;
                for ( int i = 0; i < comboPlatform.Items.Count; i++ ) {
                    string title = (string)comboPlatform.Items[i];
                    if ( title == m_platform + "" ) {
                        comboPlatform.SelectedIndex = i;
                        break;
                    }
                }
            }
        }

        public int MaximumFrameRate {
            get {
                return (int)numMaximumFrameRate.Value;
            }
            set {
                numMaximumFrameRate.Value = value;
            }
        }

        public bool ScrollHorizontalOnWheel {
            get {
                return chkScrollHorizontal.Checked;
            }
            set {
                chkScrollHorizontal.Checked = value;
            }
        }

        public void ApplyLanguage() {
            Text = _( "Preference" );
            btnCancel.Text = _( "Cancel" );
            btnOK.Text = _( "OK" );
            try {
                openUtauCore.Filter = _( "Executable(*.exe)|*.exe" ) + "|" + _( "All Files(*.*)|*.*" );
            } catch {
                openUtauCore.Filter = "Executable(*.exe)|*.exe|All Files(*.*)|*.*";
            }

            folderBrowserSingers.Description = _( "Select Singer Directory" );

            #region tabSequence
            tabSequence.Text = _( "Sequence" );
            lblResolution.Text = _( "Resolution(VSTi)" );
            lblDynamics.Text = _( "Dynamics" ) + "(&D)";
            lblAmplitude.Text = _( "Vibrato Depth" ) + "(&R)";
            lblPeriod.Text = _( "Vibrato Rate" ) + "(&V)";
            lblVibratoConfig.Text = _( "Vibrato Settings" );
            lblVibratoLength.Text = _( "Default Vibrato Length" ) + "(&L)";
            groupAutoVibratoConfig.Text = _( "Auto Vibrato Settings" );
            chkEnableAutoVibrato.Text = _( "Enable Automatic Vibrato" ) + "(&E)";
            lblAutoVibratoMinLength.Text = _( "Minimum note length for Automatic Vibrato" ) + "(&M)";
            lblAutoVibratoType.Text = _( "Vibrato Type" ) + "(&T)";
            #endregion

            #region tabAudio
            //tabAudio.Text = _( "Audio Settings" );
            #endregion

            #region tabAnother
            tabAnother.Text = _( "Other Settings" );
            lblDefaultSinger.Text = _( "Default Singer" ) + "(&S)";
            lblPreSendTime.Text = _( "Pre-Send time" ) + "(&P)";
            lblWait.Text = _( "Waiting Time" ) + "(&W)";
            lblDefaultPremeasure.Text = _( "Default Pre-measure" ) + "(&M)";
            chkChasePastEvent.Text = _( "Chase Event" ) + "(&C)";
            lblSampleOutput.Text = _( "Playback Sample Sound" );
            chkEnableSampleOutput.Text = _( "Enable" ) + "(&E)";
            lblTiming.Text = _( "Timing" );
            lblPreSendTimeSample.Text = _( "Pre-Send Time for sample sound" ) + "(&G)";
            #endregion

            #region tabAppearance
            tabAppearance.Text = _( "Appearance" );
            groupFont.Text = _( "Font" );
            labelMenu.Text = _( "Menu / Lyrics" );
            labelScreen.Text = _( "Screen" );
            labelCounter.Text = _( "Counter" );
            lblLanguage.Text = _( "UI Language" );
            btnChangeCounterFont.Text = _( "Change" );
            btnChangeMenuFont.Text = _( "Change" );
            btnChangeScreenFont.Text = _( "Change" );
            lblTrackHeight.Text = _( "Track Height (pixel)" );
            groupVisibleCurve.Text = _( "Visible Control Curve" );
            #endregion

            #region tabOperation
            tabOperation.Text = _( "Operation" );
            labelWheelOrder.Text = _( "Mouse wheel Rate" );
            chkCursorFix.Text = _( "Fix Play Cursor to Center" );
            chkScrollHorizontal.Text = _( "Horizontal Scroll when Mouse wheel" );
            lblMaximumFrameRate.Text = _( "Maximum Frame Rate" );
            chkKeepLyricInputMode.Text = _( "Keep Lyric Input Mode" );
            lblMouseHoverTime.Text = _( "Waiting Time for Preview" );
            lblMilliSecond.Text = _( "milli second" );
            chkPlayPreviewWhenRightClick.Text = _( "Play Preview On Right Click" );
            chkCurveSelectingQuantized.Text = _( "Enable Quantize for Curve Selecting" );
            lblMidiInPort.Text = _( "MIDI In Port Number" );

            groupPianoroll.Text = _( "Piano Roll" );
            groupMisc.Text = _( "Misc" );
            #endregion

            #region tabPlatform
            tabPlatform.Text = _( "Platform" );
            
            groupPlatform.Text = _( "Platform" );
            lblPlatform.Text = _( "Current Platform" );
            chkCommandKeyAsControl.Text = _( "Use Command key as Control key" );
            chkUseCustomFileDialog.Text = _( "Use Custom File Dialog" );
            chkTranslateRoman.Text = _( "Translate Roman letters into Kana" );

            groupUtauCores.Text = _( "UTAU Cores" );
            chkInvokeWithWine.Text = _( "Invoke UTAU cores with Wine" );
            #endregion

            #region tabUtauSingers
            tabUtauSingers.Text = _( "UTAU Singers" );
            columnHeaderProgramChange.Text = _( "Program Change" );
            columnHeaderName.Text = _( "Name" );
            columnHeaderPath.Text = _( "Path" );
            btnAdd.Text = _( "Add" );
            btnRemove.Text = _( "Remove" );
            btnUp.Text = _( "Up" );
            btnDown.Text = _( "Down" );
            #endregion
        }

        public static string _( string id ) {
            return Messaging.GetMessage( id );
        }

        public string Language {
            get {
                int index = comboLanguage.SelectedIndex;
                if ( 0 <= index && index < comboLanguage.Items.Count ) {
                    string title = (string)comboLanguage.Items[index];
                    if ( title == "Default" ) {
                        return "";
                    } else {
                        return title;
                    }
                } else {
                    return "";
                }
            }
        }

        public ClockResolution ControlCurveResolution {
            get {
                int count = -1;
                int index = comboDynamics.SelectedIndex;
                foreach ( ClockResolution vt in ClockResolution.GetEnumerator() ) {
                    count++;
                    if ( count == index ) {
                        return vt;
                    }
                }
                comboDynamics.SelectedIndex = 0;
                return ClockResolution.L30;
            }
            set {
                int count = -1;
                foreach ( ClockResolution vt in ClockResolution.GetEnumerator() ) {
                    count++;
                    if ( vt.Equals( value ) ) {
                        comboDynamics.SelectedIndex = count;
                        break;
                    }
                }
            }
        }

        public int PreSendTime {
            get {
                return (int)numPreSendTime.Value;
            }
            set {
                numPreSendTime.Value = value;
            }
        }

        public int PreMeasure {
            get {
                return comboDefaultPremeasure.SelectedIndex + 1;
            }
            set {
                comboDefaultPremeasure.SelectedIndex = value - 1;
            }
        }

        public bool EnableAutoVibrato {
            get {
                return chkEnableAutoVibrato.Checked;
            }
            set {
                chkEnableAutoVibrato.Checked = value;
            }
        }

        public VibratoType AutoVibratoType {
            get {
                int count = -1;
                int index = comboAutoVibratoType.SelectedIndex;
                foreach ( VibratoType vt in Enum.GetValues( typeof( VibratoType ) ) ) {
                    count++;
                    if ( count == index ) {
                        return vt;
                    }
                }
                comboAutoVibratoType.SelectedIndex = 0;
                return VibratoType.NormalType1;
            }
            set {
                int count = -1;
                foreach ( VibratoType vt in Enum.GetValues( typeof( VibratoType ) ) ) {
                    count++;
                    if ( vt == value ) {
                        comboAutoVibratoType.SelectedIndex = count;
                        break;
                    }
                }
            }
        }

        public AutoVibratoMinLength AutoVibratoMinimumLength {
            get {
                int count = -1;
                int index = comboAutoVibratoMinLength.SelectedIndex;
                foreach ( AutoVibratoMinLength avml in Enum.GetValues( typeof( AutoVibratoMinLength ) ) ) {
                    count++;
                    if ( count == index ) {
                        return avml;
                    }
                }
                comboAutoVibratoMinLength.SelectedIndex = 0;
                return AutoVibratoMinLength.L1;
            }
            set {
                int count = -1;
                foreach ( AutoVibratoMinLength avml in Enum.GetValues( typeof( AutoVibratoMinLength ) ) ) {
                    count++;
                    if ( avml == value ) {
                        comboAutoVibratoMinLength.SelectedIndex = count;
                        break;
                    }
                }
            }
        }

        public DefaultVibratoLength DefaultVibratoLength {
            get {
                int count = -1;
                int index = comboVibratoLength.SelectedIndex;
                foreach ( DefaultVibratoLength vt in Enum.GetValues( typeof( DefaultVibratoLength ) ) ) {
                    count++;
                    if ( index == count ) {
                        return vt;
                    }
                }
                comboVibratoLength.SelectedIndex = 1;
                return DefaultVibratoLength.L66;
            }
            set {
                int count = -1;
                foreach ( DefaultVibratoLength dvl in Enum.GetValues( typeof( DefaultVibratoLength ) ) ) {
                    count++;
                    if ( dvl == value ) {
                        comboVibratoLength.SelectedIndex = count;
                        break;
                    }
                }
            }
        }

        public bool CursorFixed {
            get {
                return chkCursorFix.Checked;
            }
            set {
                chkCursorFix.Checked = value;
            }
        }

        public int WheelOrder {
            get {
                return (int)numericUpDownEx1.Value;
            }
            set {
                if ( value < numericUpDownEx1.Minimum ) {
                    numericUpDownEx1.Value = numericUpDownEx1.Minimum;
                } else if ( numericUpDownEx1.Maximum < value ) {
                    numericUpDownEx1.Value = numericUpDownEx1.Maximum;
                } else {
                    numericUpDownEx1.Value = value;
                }
            }
        }

        public Font ScreenFont {
            get {
                return m_screen_font;
            }
            set {
                m_screen_font = value;
                labelScreenFontName.Text = m_screen_font.Name;
            }
        }

        public Font CounterFont {
            get {
                return m_counter_font;
            }
            set {
                m_counter_font = value;
                labelCounterFontName.Text = m_counter_font.Name;
            }
        }

        public Font BaseFont {
            get {
                return m_base_font;
            }
            set {
                m_base_font = value;
                labelMenuFontName.Text = m_base_font.Name;
                UpdateFonts( m_base_font.Name );
            }
        }

        public string DefaultSingerName {
            get {
                if ( comboDefualtSinger.SelectedIndex >= 0 ) {
                    return m_program_change[comboDefualtSinger.SelectedIndex];
                } else {
                    return "Miku";
                }
            }
            set {
                int index = -1;
                for ( int i = 0; i < m_program_change.Count; i++ ) {
                    if ( m_program_change[i] == value ) {
                        index = i;
                        break;
                    }
                }
                if ( index >= 0 ) {
                    comboDefualtSinger.SelectedIndex = index;
                }
            }
        }

        private void btnChangeMenuFont_Click( object sender, EventArgs e ) {
            fontDialog.Font = BaseFont;
            if ( fontDialog.ShowDialog() == DialogResult.OK ) {
                BaseFont = (Font)fontDialog.Font.Clone();
            }
        }

        private void btnOK_Click( object sender, EventArgs e ) {
            this.DialogResult = DialogResult.OK;
        }

        private void btnChangeScreenFont_Click( object sender, EventArgs e ) {
            fontDialog.Font = ScreenFont;
            if ( fontDialog.ShowDialog() == DialogResult.OK ) {
                ScreenFont = (Font)fontDialog.Font.Clone();
            }
        }

        private void btnChangeCounterFont_Click( object sender, EventArgs e ) {
            fontDialog.Font = CounterFont;
            if ( fontDialog.ShowDialog() == DialogResult.OK ) {
                CounterFont = (Font)fontDialog.Font.Clone();
            }
        }

        void UpdateFonts( string font_name ) {
            if ( font_name == "" ) {
                return;
            }
            Font font = new Font( font_name, tabSequence.Font.Size, tabSequence.Font.Unit );
            foreach ( Control ctrl in this.Controls ) {
                ctrl.Font = font;
            }
        }

        private void comboPlatform_SelectedIndexChanged( object sender, EventArgs e ) {
            string title = (string)comboPlatform.SelectedItem;
            foreach ( Platform p in Enum.GetValues( typeof( Platform ) ) ) {
                if ( title == p + "" ) {
                    m_platform = p;
                    chkCommandKeyAsControl.Enabled = p != Platform.Windows;
                    chkUseCustomFileDialog.Enabled = p != Platform.Windows;
                    break;
                }
            }
        }

        public bool CommandKeyAsControl {
            get {
                return chkCommandKeyAsControl.Checked;
            }
            set {
                chkCommandKeyAsControl.Checked = value;
            }
        }

        private void btnResampler_Click( object sender, EventArgs e ) {
            if ( txtResampler.Text != "" && Directory.Exists( Path.GetDirectoryName( txtResampler.Text ) ) ) {
                openUtauCore.InitialDirectory = Path.GetDirectoryName( txtResampler.Text );
            }
            openUtauCore.FileName = "resampler.exe";
            DialogResult dr = DialogResult.Cancel;
            if ( AppManager.EditorConfig.UseCustomFileDialog ) {
                using ( FileDialog fd = new FileDialog( FileDialog.DialogMode.Open ) ) {
                    fd.InitialDirectory = openUtauCore.InitialDirectory;
                    fd.Filter = openUtauCore.Filter;
                    dr = fd.ShowDialog();
                    if ( dr == DialogResult.OK ) {
                        openUtauCore.FileName = fd.FileName;
                    }
                }
            } else {
                dr = openUtauCore.ShowDialog();
            }
            if ( dr == DialogResult.OK ) {
                txtResampler.Text = openUtauCore.FileName;
            }
        }

        private void btnWavtool_Click( object sender, EventArgs e ) {
            if ( txtWavtool.Text != "" && Directory.Exists( Path.GetDirectoryName( txtWavtool.Text ) ) ) {
                openUtauCore.InitialDirectory = Path.GetDirectoryName( txtWavtool.Text );
            }
            openUtauCore.FileName = "wavtool.exe";
            DialogResult dr = DialogResult.Cancel;
            if ( AppManager.EditorConfig.UseCustomFileDialog ) {
                using ( FileDialog fd = new FileDialog( FileDialog.DialogMode.Open ) ) {
                    fd.InitialDirectory = openUtauCore.InitialDirectory;
                    fd.Filter = openUtauCore.Filter;
                    dr = fd.ShowDialog();
                    if ( dr == DialogResult.OK ) {
                        openUtauCore.FileName = fd.FileName;
                    }
                }
            } else {
                dr = openUtauCore.ShowDialog();
            }
            if ( dr == DialogResult.OK ) {
                txtWavtool.Text = openUtauCore.FileName;
            }
        }

        public string PathResampler {
            get {
                return txtResampler.Text;
            }
            set {
                txtResampler.Text = value;
            }
        }

        public string PathWavtool {
            get {
                return txtWavtool.Text;
            }
            set {
                txtWavtool.Text = value;
            }
        }

        public List<SingerConfig> UtauSingers {
            get {
                return m_utau_singers;
            }
            set {
                m_utau_singers.Clear();
                for ( int i = 0; i < value.Count; i++ ) {
                    m_utau_singers.Add( (SingerConfig)value[i].Clone() );
                }
                UpdateUtauSingerList();
            }
        }

        private void UpdateUtauSingerList() {
            listSingers.Items.Clear();
            for ( int i = 0; i < m_utau_singers.Count; i++ ) {
                m_utau_singers[i].Program = i;
                listSingers.Items.Add( new ListViewItem( new string[] { m_utau_singers[i].Program.ToString(),
                                                                        m_utau_singers[i].VOICENAME, 
                                                                        m_utau_singers[i].VOICEIDSTR } ) );
            }
        }

        private void btnAdd_Click( object sender, EventArgs e ) {
            if ( folderBrowserSingers.ShowDialog() == DialogResult.OK ) {
                string dir = folderBrowserSingers.SelectedPath;
                SingerConfig sc = new SingerConfig();
                string character = Path.Combine( dir, "character.txt" );
                string name = "";
                sc.VOICEIDSTR = dir;
                if ( File.Exists( character ) ) {
                    using ( cp932reader sr = new cp932reader( character ) ) {
                        while ( sr.Peek() >= 0 ) {
                            string line = sr.ReadLine();
                            string[] spl = line.Split( '=' );
                            if ( spl.Length >= 2 ) {
                                if ( spl[0].ToLower() == "name" ) {
                                    name = spl[1];
                                    break;
                                }
                            }
                        }
                    }
                }
                if ( name == "" ) {
                    name = "Unknown";
                }
                sc.VOICENAME = name;
                m_utau_singers.Add( sc );
                UpdateUtauSingerList();
            }
        }

        private int UtauSingersSelectedIndex {
            get {
                int count = listSingers.SelectedIndices.Count;
                if ( count <= 0 ) {
                    return -1;
                } else {
                    return listSingers.SelectedIndices[0];
                }
            }
        }

        private void listSingers_SelectedIndexChanged( object sender, EventArgs e ) {
            int index = UtauSingersSelectedIndex;
            if ( index < 0 ) {
                btnRemove.Enabled = false;
                btnUp.Enabled = false;
                btnDown.Enabled = false;
            } else {
                btnRemove.Enabled = true;
                btnUp.Enabled = 0 <= index - 1 && index - 1 < m_utau_singers.Count;
                btnDown.Enabled = 0 <= index + 1 && index + 1 < m_utau_singers.Count;
            }
        }

        private void btnRemove_Click( object sender, EventArgs e ) {
            int index = UtauSingersSelectedIndex;
            if ( 0 <= index && index < m_utau_singers.Count ) {
                m_utau_singers.RemoveAt( index );
            }
            UpdateUtauSingerList();
        }

        private void btnDown_Click( object sender, EventArgs e ) {
            int index = UtauSingersSelectedIndex;
#if DEBUG
            Common.DebugWriteLine( "Preference.btnDown_Click; index=" + index );
#endif
            if ( 0 <= index && index + 1 < m_utau_singers.Count ) {
                SingerConfig buf = (SingerConfig)m_utau_singers[index].Clone();
                m_utau_singers[index] = (SingerConfig)m_utau_singers[index + 1].Clone();
                m_utau_singers[index + 1] = buf;
                UpdateUtauSingerList();
                listSingers.Items[index + 1].Selected = true;
            }
        }

        private void btnUp_Click( object sender, EventArgs e ) {
            int index = UtauSingersSelectedIndex;
#if DEBUG
            Common.DebugWriteLine( "Preference.btnUp_Click; index=" + index );
#endif
            if ( 0 <= index - 1 && index < m_utau_singers.Count ) {
                SingerConfig buf = (SingerConfig)m_utau_singers[index].Clone();
                m_utau_singers[index] = (SingerConfig)m_utau_singers[index - 1].Clone();
                m_utau_singers[index - 1] = buf;
                UpdateUtauSingerList();
                listSingers.Items[index - 1].Selected = true;
            }
        }
    }

}
